#https://developer.aitrios.sony-semicon.com/edge-ai-sensing/documents/pytorch-model-deployment-guide?version=2025-04-23&progLang=#_quantization_of_the_model
import torch
import torch.nn as nn
import torchvision.transforms as transforms
import torchvision.datasets as datasets
from torch.utils.data import DataLoader

import model_compression_toolkit as mct
from model_compression_toolkit.core import QuantizationErrorMethod

from torchvision.models import mobilenet_v2
import argparse
import os         # takuya

def count_directories_one_level(path):            # takuya
    return sum(                                   # takuya
        os.path.isdir(os.path.join(path, entry))  # takuya
        for entry in os.listdir(path)             # takuya
    )                                             # takuya

#NUMBER_OF_CLASSES = 2  # takuya
NUMBER_OF_CLASSES = count_directories_one_level("./train")  # takuya

if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    parser.add_argument('--model_path', type=str, default='./weight.pth', help='The path to the PyTorch model')
#    parser.add_argument('--dataset_path', type=str, default='./dataset/train', help='The path to the calibration dataset') # takuya
    parser.add_argument('--dataset_path', type=str, default='./train', help='The path to the calibration dataset') # takuya
    parser.add_argument('--quantized_model_path', type=str, default='quantized.onnx', help='The path to the quantized model')
    args = parser.parse_args()

    #<1>  Load a floating-point PyTorch model.
    model = mobilenet_v2()
    num_ftrs = model.classifier[1].in_features
    model.classifier[1] = nn.Linear(num_ftrs, NUMBER_OF_CLASSES)
    model.classifier = nn.Sequential(
                        model.classifier[0],
                        model.classifier[1],
                        nn.Softmax(dim=1)
                    )
    model.load_state_dict(torch.load(args.model_path, map_location=torch.device('cpu')))

    #<2> Load a calibration dataset for quantization.
    #    The calibration dataset is normalized to match the normalization used during training.
    dataset = datasets.ImageFolder(
        args.dataset_path,
        transform = transforms.Compose([
            transforms.Resize(size=(224,224)),
            transforms.ToTensor()
        ]) )

    batch_size = 32
    train_loader = torch.utils.data.DataLoader(dataset, batch_size=batch_size, shuffle=False)
    image_data_loader = iter(train_loader)

    #<3> Create a representative dataset generator
    n_iter=len(train_loader)
    def representative_data_gen() -> list:
        ds_iter = iter(train_loader)
        for _ in range(n_iter):
            yield [next(ds_iter)[0]]

    #<4> Set a configuration.
    q_config = mct.core.QuantizationConfig(activation_error_method=QuantizationErrorMethod.MSE,
                                       weights_error_method=QuantizationErrorMethod.MSE,
                                       weights_bias_correction=True,
                                       shift_negative_activation_correction=True,
                                       z_threshold=16)
    tpc = mct.get_target_platform_capabilities("pytorch", 'imx500', target_platform_version='v1')
    ptq_config = mct.core.CoreConfig(quantization_config=q_config)

    #<5> Quantize the floating-point PyTorch model to the 8-bit integer PyTotch model.
    quantized_model, quantization_info = mct.ptq.pytorch_post_training_quantization(in_module=model,
        representative_data_gen=representative_data_gen,
        core_config=ptq_config,
        target_platform_capabilities=tpc)

    #<6> Save the integer model as an ONNX model.
    mct.exporter.pytorch_export_model(model=quantized_model,
                                      save_model_path=args.quantized_model_path,
                                      repr_dataset=representative_data_gen)